package dev.svitan.plugins

import dev.svitan.routes.routeAuth
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.plugins.NotFoundException
import io.ktor.server.plugins.requestvalidation.*
import io.ktor.server.plugins.statuspages.*
import io.ktor.server.response.*
import io.ktor.server.routing.get
import io.ktor.server.routing.routing

fun Application.configureRouting() {
    install(RequestValidation) {
        validate<String> { bodyText ->
            if (!bodyText.startsWith("Hello"))
                ValidationResult.Invalid("Body text should start with 'Hello'")
            else ValidationResult.Valid
        }
    }

    install(StatusPages) {
        exception<Throwable> { call, cause ->
            when (cause) {
                is IllegalArgumentException -> call.respond(
                    status = HttpStatusCode.BadRequest,
                    message = cause.message!!
                )

                is NotFoundException -> call.respond(
                    status = HttpStatusCode.NotFound,
                    message = cause.message ?: ""
                )

                else -> call.respond(
                    status = HttpStatusCode.InternalServerError,
                    message = cause.message ?: ""
                )
            }
        }
    }

    routing {
        get("/") {
            call.respond("Hello World!")
        }
    }

    routeAuth()
}
